@@ -20,10 +20,13 @@
 #ifndef DEFS_H
 #define DEFS_H
 #include <stdio.h>
-#include <iostream.h>
+#include "iostream"
 #include <cassert>
 #include <limits.h>
 #include <float.h>
+
+using std::cout;
+
 template <class T>
 inline void zap(T & x)
 {
@@ -39,8 +42,8 @@ inline void zaparr(T & x)
      delete [] x;
      x = NULL;
 }
-static char* release_no="0.5alpha";
-static char* release="Freecycle 0.5alpha";
+#define RELEASE_NO "0.6alpha"
+#define RELEASE_NAME "Freecycle 0.6alpha"
 static const int RTTI_BEATLINE=1001;
 static const int RTTI_BPMLINE=1002;
 static const int RTTI_LOOPMARKER=1003;
@@ -52,6 +55,7 @@ static const int RTTI_SPLINELINE=1008;
 static const int RTTI_LOCKEDBEATRECT=1009;
 static const int RTTI_LADSPAVALUERECT=1010;
 static const int RTTI_KEYBOARDRECT=1011;
+static const int RTTI_PITCHSHIFTRECT=1012;
 
 
 static const double BEAT_DISTANCE_TRESHOLD=128;
