@@ -44,10 +44,7 @@
 #include <qwhatsthis.h>
 #include <qvbox.h>
 #include <qcanvas.h>
-#include <iostream.h>
-#include 	<fcntl.h>
-#include 	<sys/ioctl.h>
-#include 	<sys/soundcard.h>
+#include <iostream>
 #include 	<string.h>
 #include <unistd.h>
 #include <qspinbox.h>
@@ -58,20 +55,24 @@
 #include "pics/icons.h"
 
 Freecycle::Freecycle()
-    : QMainWindow( 0, release, WDestructiveClose )
+    : QMainWindow( 0, RELEASE_NAME, WDestructiveClose )
 {
+    loadingProject=FALSE;
     QDir freecycle_dir(QDir::homeDirPath().append("/.freecycle"));
     if (!freecycle_dir.exists()) freecycle_dir.mkdir(freecycle_dir.absPath());
     helpManager=new HelpManager(freecycle_dir.path());
 
     soundManager=new SoundManager();
+    soundManager->setHasEnvelope(TRUE);
     soundManager->loadEffects();
     fileManager=new FileManager();
     soundManager->setFileManager(fileManager);
     midiManager=new MidiManager();
     akpManager=new AKPManager();
+#ifdef HAS_LIBINSTPATCH
     sf2Manager=new SF2Manager();
     gigaManager=new GigaManager();
+#endif
     QPixmap openIcon, saveIcon, printIcon;
     openIcon = QPixmap( fileopen );
     saveIcon = QPixmap( filesave );
@@ -127,14 +128,16 @@ Freecycle::Freecycle()
     configureAnalysisPopupMenu->insertItem(tr("(Aubio) Phase based"),this,SLOT(setPhaseDomain()),0,4);
     configureAnalysisPopupMenu->insertItem(tr("(Aubio) Spectral difference"),this,SLOT(setSpectralDomain()),0,5);
     configureAnalysisPopupMenu->insertItem(tr("(Aubio) High Frequency Content"),this,SLOT(setHFCDomain()),0,6);
-    configureAnalysisPopupMenu->setItemEnabled(5,FALSE);
+    configureAnalysisPopupMenu->insertItem(tr("(Aubio) Modified Kullback Liebler"),this,SLOT(setMKLDomain()),0,7);
+    configureAnalysisPopupMenu->insertItem(tr("(Aubio) Weighted Kullback Liebler"),this,SLOT(setKLDomain()),0,8);
+    //configureAnalysisPopupMenu->setItemEnabled(5,FALSE);
 
 #endif // HAS_AUBIO
     configureAnalysisPopupMenu->setItemChecked(1,TRUE);
     configureAnalysisPopupMenu->insertSeparator();
 #ifdef HAS_AUBIO
-    configureAnalysisPopupMenu->insertItem(tr("DerivateSignal"),this,SLOT(setDerivateSignal()),0,7);
-    configureAnalysisPopupMenu->setItemChecked(7,FALSE);
+    configureAnalysisPopupMenu->insertItem(tr("DerivateSignal"),this,SLOT(setDerivateSignal()),0,9);
+    configureAnalysisPopupMenu->setItemChecked(9,FALSE);
 #else
     configureAnalysisPopupMenu->insertItem(tr("DerivateSignal"),this,SLOT(setDerivateSignal()),0,3);
     configureAnalysisPopupMenu->setItemChecked(3,FALSE);
@@ -152,9 +155,19 @@ Freecycle::Freecycle()
     configurationPopupMenu->insertItem(tr("Configure FFT ..."),this,SLOT(configureFFT()));
     ampDomainDialog=new AmpDomainDialog(soundManager,this);
     configurationPopupMenu->insertItem(tr("Configure Amplitude domain ..."),this,SLOT(configureAmpDomain()));
-    midiInput=new MidiInput();
-    midiConfigDialog=new MidiConfigDialog(midiInput,this);
-    configurationPopupMenu->insertItem(tr("Configure MIDI ..."),this,SLOT(configureMIDI()));
+    
+    #ifdef HAS_ALSA
+        midiInput=MidiInput::instantiateAlsaSeq();
+    #else
+        #ifdef HAS_PORTMIDI
+            midiInput=MidiInput::instantiatePortMidi();
+        #endif
+    #endif
+    #ifdef HAS_MIDI
+        midiConfigDialog=new MidiConfigDialog(midiInput,this);
+        configurationPopupMenu->insertItem(tr("Configure MIDI ..."),this,SLOT(configureMIDI()));
+    #endif
+    
 
 
     menuBar()->insertItem( tr("&Configuration"), configurationPopupMenu,3);
@@ -163,7 +176,7 @@ Freecycle::Freecycle()
 
 
 
-    resize( 800, 600 );
+    resize( 1000, 800 );
 
 
 
@@ -190,12 +203,12 @@ Freecycle::Freecycle()
     bpmModifDropDown=new BPMModifierDropDown(topBox);
     connect(bpmModifDropDown,SIGNAL(valueChanged()),this,SLOT(bpmModifierValueChanged()));
 
-
-    resampleButton=new QPushButton("time stretch to:",topBox,"resample button");
-
-    newBPM=new FloatSpinBox(topBox);
-    newBPM->setMinValue(300);
-    newBPM->setMaxValue(3000);
+    #ifdef HAS_SOUNDTOUCH
+        resampleButton=new QPushButton("time stretch to:",topBox,"resample button");
+        newBPM=new FloatSpinBox(topBox);
+        newBPM->setMinValue(300);
+        newBPM->setMaxValue(3000);
+    #endif
 
 
 
@@ -258,7 +271,7 @@ Freecycle::Freecycle()
     tresholdSlider->setTickInterval(10);
     tresholdSlider->setLineStep(1);
     tresholdSlider->setPageStep(10);
-    tresholdSlider->setMaximumWidth(200);
+    tresholdSlider->setFixedWidth(200);
     connect(tresholdSlider, SIGNAL(sliderReleased()), SLOT(changeTreshold()) );
 
     analyseWindowDivider=new QSlider(beatLinesControl);
@@ -287,11 +300,6 @@ Freecycle::Freecycle()
     lockBeatLinesButton->setPixmap(QPixmap(qembed_findImage("encrypted-22")));
 
     envelopeControllerWidget=new EnvelopeControllerWidget(playAndBeatLinesControls);
-    envelopeControllerWidget->setFixedHeight(50);
-    envelopeControllerWidget->setFixedWidth(350);
-    envelopeControllerWidget->setFrameStyle(QFrame::Panel | QFrame::Raised);
-    envelopeControllerWidget->setSpacing(5);
-    envelopeControllerWidget->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Fixed);
     //envelopeControllerWidget->hide(); //for now..
     connect(envelopeControllerWidget,SIGNAL(envelopeChanged(EnvelopeData* )),
             soundManager,SLOT(envelopeChanged(EnvelopeData* )));
@@ -317,6 +325,7 @@ Freecycle::Freecycle()
 
     drawSoundWidget=new DrawSoundWidget(big_wave_and_env,"central wave",soundManager);
     drawSoundWidget->enableClipper(TRUE);
+    
     connect(sceneFrame,SIGNAL(sceneSwitchedS(int)),drawSoundWidget,SLOT(sceneSwitched(int )));
     connect(sceneFrame,SIGNAL(assignToSceneS(int)),drawSoundWidget,SLOT(assignToScene(int )));
 
@@ -326,7 +335,9 @@ Freecycle::Freecycle()
     bpmLines->setDisabled(TRUE);
     connect(drawSoundWidget,SIGNAL(hasLockedBeatLines(bool)),bpmLines,SLOT(setEnabled(bool)));
     
-    midiInput->addMidiMonitor(drawSoundWidget);
+    #ifdef HAS_MIDI
+        midiInput->addMidiMonitor(drawSoundWidget);
+    #endif
 
     //replacedWaveHolder=new ReplacedWaveHolder(big_wave_and_env);
     //replacedWaveHolder->setMaximumHeight(55);
@@ -389,14 +400,15 @@ Freecycle::Freecycle()
 
 
 
-
-    connect(newBPM,
+    #ifdef HAS_SOUNDTOUCH
+        connect(newBPM,
                     SIGNAL(valueChanged(const QString &)),
                     drawSoundWidget,SLOT(computeNewBPMLines(const QString &)));
-
-
-    connect(resampleButton,
+        connect(resampleButton,
                       SIGNAL(clicked()),drawSoundWidget,SLOT(resample()));
+    #endif
+
+
     MySplitter *small_waveAndSpectrum = new MySplitter(Qt::Horizontal, big_and_small_wave );
     small_waveAndSpectrum->setOpaqueResize(TRUE);
 
@@ -423,8 +435,10 @@ Freecycle::Freecycle()
     connect(drawSoundWidget,SIGNAL(playFromTo(long,long)),this,SLOT(playFromTo(long,long)));
     connect(drawSoundWidget,SIGNAL(stopPlay()),this,SLOT(stopPlay()));
     
-    connect(drawSoundWidget,SIGNAL(midiAssChanged(QValueList <beatline_data>)),
+    #ifdef HAS_MIDI
+        connect(drawSoundWidget,SIGNAL(midiAssChanged(QValueList <beatline_data>)),
             midiInput, SLOT(setBeatlineData(QValueList <beatline_data>)));
+    #endif
 
 
     spectrumView=new SpectrumView(small_waveAndSpectrum);
@@ -441,7 +455,15 @@ Freecycle::Freecycle()
     soundPlayer->addSoundMonitor(drawSoundWidget); //Why QThread isn't QObject ?!?
     soundPlayer->addSoundMonitor(spectrumView);        //Why QThread isn't QObject ?!?
     soundPlayer->addSoundMonitor(scopeView);        //Why QThread isn't QObject ?!?
-    midiInput->setSoundPlayer(soundPlayer);
+    #ifdef HAS_MIDI
+        midiInput->setSoundPlayer(soundPlayer);
+    #endif
+    #ifdef HAS_ALSA
+        if(soundPlayer->output==SoundPlayer::OUTPUT_ALSA){
+            alsaConfigDialog=new AlsaConfigDialog(soundPlayer,this);
+            configurationPopupMenu->insertItem(tr("Configure ALSA ..."),this,SLOT(configureALSA()));
+        }
+    #endif
 
 
     centralLayout->hide();
@@ -494,7 +516,7 @@ Freecycle::Freecycle()
     projectManager->setLoopMarkers(drawSoundWidget->getLeftLoopMarker(),
                                                                     drawSoundWidget->getRightLoopMarker());
 
-    connect(projectManager,SIGNAL(loadFile(const QString& )), this, SLOT(load(const QString& )));
+    connect(projectManager,SIGNAL(loadFile(const QString&)), this, SLOT(load(const QString&)));
     connect(projectManager,SIGNAL(envelopeDataChanged()),
                 envelopeControllerWidget, SLOT(envDataChanged()));
     connect(projectManager,SIGNAL(addLadspa(int)),this,SLOT(addEffectFrame(int)));
@@ -527,8 +549,10 @@ Freecycle::Freecycle()
     
     
     
-    midiInputThread=new MidiInputThread(midiInput);
-    midiInputThread->start();
+    #ifdef HAS_MIDI
+        midiInputThread=new MidiInputThread(midiInput);
+        midiInputThread->start();
+    #endif
 
     connect(drawSoundWidget,SIGNAL(beatLinesChanged(QValueList< beatline_data > )),
         soundManager,SLOT(beatLinesChanged(QValueList< beatline_data > )));
@@ -599,51 +623,18 @@ void Freecycle::choose()
 }
 
 
-void Freecycle::load( const QString &fileName )
+void Freecycle::load( const QString &fileName)
 {
+        drawSoundWidget->unlockBeatLines(true);
         QFile f( fileName );
         if ( !f.open( IO_ReadOnly ) )	return;
-        setCaption(release+QString(" - ")+fileName);
-
-        removeEffect();
-        fileManager->setFilename(fileName);
-        soundManager->loadFile();
-        soundManager->setHasEnvelope(FALSE);
-        soundManager->setHasEnvelope(TRUE);
-        soundPlayer->initLoop();
-        soundPlayer->setPosition(0);
-
-        zoomSlider->setValue(1);
-
-        long zoomRatioDSW=(long)(soundManager->getFrames()/
-                                                drawSoundWidget->visibleWidth());
-        long zoomRatioDASW=(long)(soundManager->getFrames()/
-                                                drawAllSoundWidget->visibleWidth());
-
-        drawSoundWidget->setZoomRatio(zoomRatioDSW);
-        drawAllSoundWidget->setZoomRatio(zoomRatioDASW);
-
-        drawSoundWidget->fileLoaded();
-        drawAllSoundWidget->fileLoaded();
-
-        statusBar()->message( tr("Loaded document %1").arg(fileName), 2000 );
-
-
-        samplerateLabel->setText(tr("samplerate: %1 Hz").arg(soundManager->getRate()));
-        nbframesLabel->setText(tr("frames: %1").arg(soundManager->getFrames()));
-        channelsLabel->setText(tr("channels: %1").arg(soundManager->getChannels()));
-        zoomRatioLabel->setText(tr("zoom: %1 - %2").arg(zoomRatioDSW).arg(1));
-        drawSoundWidget->unlockBeatLines();
-
-
-
-
-
-
-        zoomSlider->setMaxValue(soundManager->getFrames()/2);
-
-
+        setCaption(RELEASE_NAME+QString(" - ")+fileName);
 
+        if(effectFrame) effectFrame->reinit();
+        if(!topBox->isVisible()) topBox->show();
+        if(!playAndBeatLinesControls->isVisible()) playAndBeatLinesControls->show();
+        if(!zoomBox->isVisible()) zoomBox->show();
+        if(!centralLayout->isVisible()) centralLayout->show();
         menuBar()->setItemEnabled(1,TRUE);
         menuBar()->setItemEnabled(2,TRUE);
         menuBar()->setItemEnabled(3,TRUE);
@@ -656,14 +647,37 @@ void Freecycle::load( const QString &fileName )
         filePopupMenu->setItemEnabled(idsaveprojectasdialog,TRUE);
         filePopupMenu->setItemEnabled(idsaveprojectdialog,TRUE);
         filePopupMenu->setItemEnabled(idsaveloopmidifile,TRUE);
-
-        if(effectFrame) effectFrame->reinit();
-        if(!topBox->isVisible()) topBox->show();
-        if(!playAndBeatLinesControls->isVisible()) playAndBeatLinesControls->show();
-        if(!zoomBox->isVisible()) zoomBox->show();
-        if(!centralLayout->isVisible()) centralLayout->show();
         soundPlayer->setPosition(0);
-        soundManager->envelopeChanged(envelopeControllerWidget->getEnvData());
+
+        if(soundManager->getHasLADSPA()) removeEffect();
+        fileManager->setFilename(fileName);
+        soundManager->loadFile();
+        zoomSlider->setMaxValue(soundManager->getFrames()/2);
+        soundPlayer->initLoop();
+        soundPlayer->setPosition(0);
+        
+        
+
+        //zoomSlider->setValue(1);
+
+        long zoomRatioDSW=(long)(soundManager->getFrames()/
+                                                drawSoundWidget->visibleWidth());
+        long zoomRatioDASW=(long)(soundManager->getFrames()/
+                                                drawAllSoundWidget->visibleWidth());
+
+        drawAllSoundWidget->setZoomRatio(zoomRatioDASW,!loadingProject);
+        drawSoundWidget->putZoomRatio(zoomRatioDSW);
+        
+        if(!loadingProject){
+            drawSoundWidget->fileLoaded();
+            drawAllSoundWidget->fileLoaded();
+        }
+
+        statusBar()->message( tr("Loaded document %1").arg(fileName), 2000 );
+        samplerateLabel->setText(tr("samplerate: %1 Hz").arg(soundManager->getRate()));
+        nbframesLabel->setText(tr("frames: %1").arg(soundManager->getFrames()));
+        channelsLabel->setText(tr("channels: %1").arg(soundManager->getChannels()));
+        zoomRatioLabel->setText(tr("zoom: %1 - %2").arg(zoomRatioDSW).arg(1));
 }
 
 void Freecycle::playFromPoint(long x){
@@ -679,7 +693,8 @@ void Freecycle::playFromTo(long x1,long x2){
 }
 
 void Freecycle::play(){
-            if(soundPlayer->getOutputType()==SoundPlayer::OUTPUT_DEVDSP){
+            if(soundPlayer->getOutputType()==SoundPlayer::OUTPUT_DEVDSP ||
+              soundPlayer->getOutputType()==SoundPlayer::OUTPUT_ALSA){
                 if(!soundPlayer->running())
                     soundPlayer->start();
             }
@@ -787,18 +802,20 @@ void Freecycle::saveChunks()
             if(saveFileDialog->exportAKP()){
                 akpManager->writeAKPFile(fn,filenames,beatlinesVL);
             }
-            if(saveFileDialog->exportSF2()){
-                sf2Manager->writeSF2File(fn,filenames,beatlinesVL);
-            }
-            if(saveFileDialog->exportDLS()){
-                dlsManager->writeDLS2File(fn,filenames,beatlinesVL);
-            }
-            if(saveFileDialog->exportGiga()){
-                gigaManager->writeGigaFile(fn,filenames,beatlinesVL);
-            }
-            if(saveFileDialog->removeChunks()){
-                fileManager->deleteDir(fn);
-            }
+            #ifdef HAS_LIBINSTPATCH
+                if(saveFileDialog->exportSF2()){
+                    sf2Manager->writeSF2File(fn,filenames,beatlinesVL);
+                }
+                if(saveFileDialog->exportDLS()){
+                    dlsManager->writeDLS2File(fn,filenames,beatlinesVL);
+                }
+                if(saveFileDialog->exportGiga()){
+                    gigaManager->writeGigaFile(fn,filenames,beatlinesVL);
+                }
+                if(saveFileDialog->removeChunks()){
+                    fileManager->deleteDir(fn);
+                }
+            #endif
             statusBar()->message( tr( "File %1 saved to chunks" ).arg( fn), 2000 );
             delete filenames;
         } else {
@@ -815,6 +832,16 @@ void Freecycle::closeEvent( QCloseEvent* ce )
         if(soundPlayer->running()){
             soundPlayer->exit(0);
         }
+        #ifdef HAS_PORTMIDI
+            QProgressDialog pd("Terminating midi thread",0,2,this,"progress",TRUE);
+            cout<<"Terminating midi thread\n";
+            pd.setMinimumDuration(0);
+            pd.setProgress(0);
+            mainApplication->processEvents();
+            delete midiInputThread;
+            pd.setProgress(1);
+            cout<<"..ok\n";
+        #endif
         delete soundPlayer;
         delete fileManager;
         ce->accept();
@@ -824,7 +851,7 @@ void Freecycle::closeEvent( QCloseEvent* ce )
 
 void Freecycle::about()
 {
-    QMessageBox::information( this, tr("Freecycle alpha 0.1"),
+    QMessageBox::information( this, tr(RELEASE_NAME),
 			tr("Developped under GNU Public Licence by:"
                         "\nPredrag Viceic <viceic@net2000.ch>"
                         "\n\nCredits:\n\n"
@@ -933,29 +960,33 @@ void Freecycle::configureAmpDomain(){
 
 void Freecycle::removeEffect(){
     soundManager->setHasLadspa(false);
-    if(soundManager->getHasEnvelope())
-        soundManager->applyEnvelope();
-    else soundManager->zapInstantEBuffer();
     if(effectFrame){
-        effectFrame->close(TRUE);
-        effectFrame=0;
+            effectFrame->close(TRUE);
+            effectFrame=0;
+        }
+    if(!loadingProject){
+        if(soundManager->getHasEnvelope())
+            soundManager->applyEnvelope();
+        else soundManager->zapInstantEBuffer();
+        
+        drawSoundWidget->zapWaveCanvasBuffers();
+        drawAllSoundWidget->zapWaveCanvasBuffers();
     }
-    drawSoundWidget->zapWaveCanvasBuffers();
-    drawAllSoundWidget->zapWaveCanvasBuffers();
     projectManager->setLadspa(0);
     soundManager->getEffects()->setCurrent(0);
+    
 }
 
 void Freecycle::addEffectFrame(int uniqueId){
-
-    removeEffect();
-    soundManager->setHasLadspa(TRUE);
+    if(soundManager->getHasLADSPA()) removeEffect();
     LADSPAEffect* temp=soundManager->getEffects()->getEffect(uniqueId);
     soundManager->getEffects()->setCurrent(temp);
+    soundManager->setHasLadspa(TRUE);
     effectFrame=new EffectFrame(temp,big_and_small_wave_and_effects);
     big_and_small_wave_and_effects->setResizeMode(effectFrame,QSplitter::Stretch);
     effectFrame->setZoomRatio(drawSoundWidget->getZoomRatio());
     effectFrame->setContentsPos(drawSoundWidget->contentsX(),0);
+    effectFrame->show();
     connect(effectFrame,SIGNAL(applyMe(LADSPAEffect*)),
                     soundManager,SLOT(applyPlugin(LADSPAEffect* )));
     connect(effectFrame,SIGNAL(applyMe(LADSPAEffect*,long,long)),
@@ -975,8 +1006,7 @@ void Freecycle::addEffectFrame(int uniqueId){
 
     connect(soundManager,SIGNAL(cropped(long, long )),effectFrame,SIGNAL(cropS(long, long )));
     effectFrame->lockedBeatLinesChanged(drawSoundWidget->getLockedBeatList());
-
-    effectFrame->show();
+    
 
     //load polylines and splines, if any
     if(projectManager->getPolylines().count()>0){
@@ -998,8 +1028,7 @@ void Freecycle::addEffectFrame(int uniqueId){
     effectFrame->updateDrawModeButton();
     //end load polylines and splines, if any
 
-
-    effectFrame->apply();
+    soundManager->applyPlugin(temp);
     projectManager->setLadspa(effectFrame);
 }
 
@@ -1017,10 +1046,12 @@ void Freecycle::setAmplitudeDomain(){
     configureAnalysisPopupMenu->setItemChecked(4,FALSE);
     configureAnalysisPopupMenu->setItemChecked(5,FALSE);
     configureAnalysisPopupMenu->setItemChecked(6,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(7,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(8,FALSE);
 #endif
 }
 void Freecycle::setFrequencyDomain(){
-    std::cout<<"set freq"<<endl;
+    std::cout<<"set freq"<<"\n";
     soundManager->setAnalysisMethod(SoundManager::FREQ_DOMAIN);
     drawSoundWidget->reinitCanvas();
     configureAnalysisPopupMenu->setItemChecked(2,TRUE);
@@ -1030,11 +1061,14 @@ void Freecycle::setFrequencyDomain(){
     configureAnalysisPopupMenu->setItemChecked(4,FALSE);
     configureAnalysisPopupMenu->setItemChecked(5,FALSE);
     configureAnalysisPopupMenu->setItemChecked(6,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(7,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(8,FALSE);
 #endif
 }
-#ifdef HAS_AUBIO
+
 void Freecycle::setComplexDomain(){
-    std::cout<<"set complex"<<endl;
+     #ifdef HAS_AUBIO
+    std::cout<<"set complex"<<"\n";
     soundManager->setAnalysisMethod(SoundManager::COMPLEX_DOMAIN);
     drawSoundWidget->reinitCanvas();
     configureAnalysisPopupMenu->setItemChecked(1,FALSE);
@@ -1043,9 +1077,13 @@ void Freecycle::setComplexDomain(){
     configureAnalysisPopupMenu->setItemChecked(4,FALSE);
     configureAnalysisPopupMenu->setItemChecked(5,FALSE);
     configureAnalysisPopupMenu->setItemChecked(6,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(7,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(8,FALSE);
+    #endif
 }
 void Freecycle::setPhaseDomain(){
-    std::cout<<"set phase"<<endl;
+     #ifdef HAS_AUBIO
+    std::cout<<"set phase"<<"\n";
     soundManager->setAnalysisMethod(SoundManager::PHASE_DIFF);
     drawSoundWidget->reinitCanvas();
     configureAnalysisPopupMenu->setItemChecked(1,FALSE);
@@ -1054,9 +1092,13 @@ void Freecycle::setPhaseDomain(){
     configureAnalysisPopupMenu->setItemChecked(4,TRUE);
     configureAnalysisPopupMenu->setItemChecked(5,FALSE);
     configureAnalysisPopupMenu->setItemChecked(6,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(7,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(8,FALSE);
+    #endif
 }
 void Freecycle::setSpectralDomain(){
-    std::cout<<"set spectral"<<endl;
+     #ifdef HAS_AUBIO
+    std::cout<<"set spectral"<<"\n";
     soundManager->setAnalysisMethod(SoundManager::SPEC_DIFF);
     drawSoundWidget->reinitCanvas();
     configureAnalysisPopupMenu->setItemChecked(1,FALSE);
@@ -1065,23 +1107,60 @@ void Freecycle::setSpectralDomain(){
     configureAnalysisPopupMenu->setItemChecked(4,FALSE);
     configureAnalysisPopupMenu->setItemChecked(5,TRUE);
     configureAnalysisPopupMenu->setItemChecked(6,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(7,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(8,FALSE);
+    #endif
 }
 void Freecycle::setHFCDomain(){
-        std::cout<<"set hfc"<<endl;
-        soundManager->setAnalysisMethod(SoundManager::HFC);
-        drawSoundWidget->reinitCanvas();
-        configureAnalysisPopupMenu->setItemChecked(1,FALSE);
-        configureAnalysisPopupMenu->setItemChecked(2,FALSE);
-        configureAnalysisPopupMenu->setItemChecked(3,FALSE);
-        configureAnalysisPopupMenu->setItemChecked(4,FALSE);
-        configureAnalysisPopupMenu->setItemChecked(5,FALSE);
-        configureAnalysisPopupMenu->setItemChecked(6,TRUE);
+    #ifdef HAS_AUBIO
+    std::cout<<"set hfc"<<"\n";
+    soundManager->setAnalysisMethod(SoundManager::HFC);
+    drawSoundWidget->reinitCanvas();
+    configureAnalysisPopupMenu->setItemChecked(1,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(2,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(3,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(4,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(5,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(6,TRUE);
+    configureAnalysisPopupMenu->setItemChecked(7,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(8,FALSE);
+    #endif
 }
-#endif
+void Freecycle::setMKLDomain(){
+    #ifdef HAS_AUBIO
+    std::cout<<"set hfc"<<"\n";
+    soundManager->setAnalysisMethod(SoundManager::MKL);
+    drawSoundWidget->reinitCanvas();
+    configureAnalysisPopupMenu->setItemChecked(1,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(2,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(3,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(4,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(5,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(6,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(7,TRUE);
+    configureAnalysisPopupMenu->setItemChecked(8,FALSE);
+    #endif
+}
+void Freecycle::setKLDomain(){
+    #ifdef HAS_AUBIO
+    std::cout<<"set hfc"<<"\n";
+    soundManager->setAnalysisMethod(SoundManager::KL);
+    drawSoundWidget->reinitCanvas();
+    configureAnalysisPopupMenu->setItemChecked(1,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(2,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(3,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(4,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(5,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(6,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(7,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(8,TRUE);
+    #endif
+}
+
 
 void Freecycle::setDerivateSignal(){
 #ifdef HAS_AUBIO
-    if(configureAnalysisPopupMenu->isItemChecked(7))
+    if(configureAnalysisPopupMenu->isItemChecked(9))
 #else
     if(configureAnalysisPopupMenu->isItemChecked(3))
 #endif
@@ -1089,7 +1168,7 @@ void Freecycle::setDerivateSignal(){
     soundManager->setDerivateSignal(FALSE);
 
 #ifdef HAS_AUBIO
-    configureAnalysisPopupMenu->setItemChecked(7,FALSE);
+    configureAnalysisPopupMenu->setItemChecked(9,FALSE);
 #else
     configureAnalysisPopupMenu->setItemChecked(3,FALSE);
 #endif
@@ -1097,7 +1176,7 @@ void Freecycle::setDerivateSignal(){
     }else{
         soundManager->setDerivateSignal(TRUE);
 #ifdef HAS_AUBIO
-     configureAnalysisPopupMenu->setItemChecked(7,TRUE);
+     configureAnalysisPopupMenu->setItemChecked(9,TRUE);
 #else
        configureAnalysisPopupMenu->setItemChecked(3,FALSE);
 #endif
@@ -1244,10 +1323,7 @@ void Freecycle::loadProject()
     QString fn=NULL;
     if ( loadProjectDialog->exec() == QDialog::Accepted ){
             fn=loadProjectDialog->selectedFile();
-            projectManager->loadProject(fn);
-            setCaption(release+QString(" - ")+fn);
-            drawSoundWidget->analysisFinished(); //force bpm computation based on loaded beatlines
-            drawSoundWidget->getCanvas()->update();
+            loadProject(fn);
     }
 
 }
@@ -1258,7 +1334,19 @@ void Freecycle::loadProject()
  */
 void Freecycle::configureMIDI()
 {
-    midiConfigDialog->exec();
+#ifdef HAS_MIDI
+        midiConfigDialog->exec();
+#endif
+}
+
+/*!
+    \fn Freecycle::configureALSA()
+ */
+void Freecycle::configureALSA()
+{
+#ifdef HAS_ALSA
+        alsaConfigDialog->exec();
+#endif
 }
 
 
@@ -1310,10 +1398,16 @@ void Freecycle::saveProject()
  */
 void Freecycle::loadProject(QString fn)
 {
-    projectManager->loadProject(fn);
-    setCaption(release+QString(" - ")+fn);
-    drawSoundWidget->analysisFinished(); //force bpm computation based on loaded beatlines
-    drawSoundWidget->getCanvas()->update();
+    if(fn!=NULL){
+        soundManager->setLoadingProject(TRUE);
+        loadingProject=TRUE;
+        projectManager->loadProject(fn);
+        loadingProject=FALSE;
+        soundManager->setLoadingProject(FALSE);
+        if(!soundManager->getHasLADSPA()) soundManager->applyEnvelope();
+        setCaption(RELEASE_NAME+QString(" - ")+fn);
+        drawSoundWidget->fileLoaded();
+    }
 }
 
 
@@ -1324,3 +1418,11 @@ void Freecycle::help()
 {
     helpManager->open();
 }
+
+
+void Freecycle::setMainApplication(QApplication* theValue) {
+    mainApplication = theValue;
+    drawSoundWidget->setMainApplication(mainApplication);
+    soundPlayer->setMainApplication(mainApplication);
+    
+}
