@@ -70,42 +70,65 @@ SoundPlayer::SoundPlayer()
         position=0;
         soundManager=0;
         buffer_len=1024;
-        bufferLR=new short[buffer_len*2];
+        bufferLR_S=new short[buffer_len*2];
+        bufferLR_F=new float[buffer_len*2];
         bufferPointerL=0;
         bufferPointerR=0;
-        devdsp_audio_device=0;
+        bufferPointersLR=new (float*)[2];
+        #ifdef HAS_OSS
+            devdsp_audio_device=0;
+        #endif
         leftLoop=0;
         rightLoop=0;
         output=OUTPUT_NONE;
         playFromFileV=FALSE;
         previewFile=0;
-        savedLeftLoop=-1;
-        savedRightLoop=-1;
+        savedLeftLoop=ULONG_MAX;
+        savedRightLoop=ULONG_MAX;
         closeV=FALSE;
-        portaudio_stream=0;
+        #ifdef HAS_PORTAUDIO
+            portaudio_stream=0;
+        #endif
+        #ifdef HAS_ALSA
+            alsa_rate = 44100;
+            alsa_periods = 2;
+            alsa_periodsize = 4096;
+            alsa_pcm_name = strdup("plughw:0,0");
+        #endif
+        processedPositions=0;
 }
 
 
 SoundPlayer::~SoundPlayer()
 {
-    zaparr(bufferLR);
-    if(output==OUTPUT_DEVDSP) ::close (devdsp_audio_device) ;
-    else if(output==OUTPUT_JACK) closeJack();
-    else if(output==OUTPUT_PORTAUDIO) closePortaudio();
+    zaparr(bufferLR_S);
+    zaparr(bufferLR_F);
+    #ifdef HAS_OSS
+        if(output==OUTPUT_DEVDSP) ::close (devdsp_audio_device) ;
+    #endif
+    #ifdef HAS_JACK
+        if(output==OUTPUT_JACK) closeJack();
+    #endif
+    #ifdef HAS_PORTAUDIO
+        if(output==OUTPUT_PORTAUDIO) closePortaudio();
+    #endif
+    #ifdef HAS_ALSA
+        if(output==OUTPUT_ALSA && alsa_pcm_handle!=0) snd_pcm_close(alsa_pcm_handle);
+    #endif
 }
-
+#ifdef HAS_OSS
 void SoundPlayer::playDevDsp(){
         while(position<rightLoop && playing){
             if(position+buffer_len<rightLoop){
-                soundManager->exportMultiplexedShortBuffer(bufferLR, position,position+buffer_len);
-                    if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR, buffer_len*2);
-                    write (devdsp_audio_device,bufferLR,buffer_len*2* sizeof (short)) ;
+                soundManager->exportMultiplexedShortBuffer(bufferLR_S, position,position+buffer_len);
+                    if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR_S, buffer_len*2);
+                    write (devdsp_audio_device,bufferLR_S,buffer_len*2* sizeof (short)) ;
                     position+=buffer_len;
             }else if (position<rightLoop){
-                soundManager->exportMultiplexedShortBuffer(bufferLR, position,
+                soundManager->exportMultiplexedShortBuffer(bufferLR_S, position,
                         rightLoop);
-                    if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR, (rightLoop-position)*2);
-                    write (devdsp_audio_device,bufferLR,(rightLoop-position)*2* sizeof (short));
+                    if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR_S, (rightLoop-position)*2);
+                    write (devdsp_audio_device,bufferLR_S,(rightLoop-position)*2* sizeof (short));
                     position=rightLoop;
             }else position=rightLoop;
             positionChanged(position);
@@ -116,13 +139,18 @@ void SoundPlayer::playDevDsp(){
             resetLoop();
         }
 }
-
+#endif
 void SoundPlayer::stopPlay(){
 
         if(playing){
             playing=false;
             positionChanged(leftLoop);
-            if(output==OUTPUT_PORTAUDIO) stopPAStream();
+            #ifdef HAS_PORTAUDIO
+                if(output==OUTPUT_PORTAUDIO) stopPAStream();
+            #endif
+            #ifdef HAS_ALSA
+                if(output==OUTPUT_ALSA) snd_pcm_drop(alsa_pcm_handle);
+            #endif
         }
 }
 
@@ -140,8 +168,15 @@ void SoundPlayer::setSoundmanager(SoundManager *sm)
 }
 
 
-void SoundPlayer::positionChanged(long _pos)
+void SoundPlayer::positionChanged(unsigned long _pos)
 {
+    /*
+    if(processedPositions>=1000){
+        processedPositions=0;
+        mainApplication->processEvents(1);
+    }
+    */
+    processedPositions++;
     position=_pos;
     QObject* soundMonitor;
     for ( soundMonitor = soundMonitors.first(); soundMonitor; soundMonitor = soundMonitors.next() ){
@@ -154,6 +189,24 @@ void SoundPlayer::positionChanged(long _pos)
     }
 
 }
+#ifdef HAS_OSS
+/*!
+    \fn SoundPlayer::connect_devdsp()
+ */
+int SoundPlayer::connect_devdsp()
+{
+        fprintf (stderr, "Querying /dev/dsp.\n");
+        //devdsp_audio_device = linux_open_dsp_device (soundManager->getChannels(),
+        //                                                                soundManager->getRate()) ;
+        devdsp_audio_device = linux_open_dsp_device (2,44100) ;
+        if(devdsp_audio_device){
+            output=OUTPUT_DEVDSP;
+            fprintf (stderr, "Connected to /dev/dsp.\n");
+            return 0;
+            bufferLR_S=new short[buffer_len*2];
+        }else return 1;
+}
+
 
 int SoundPlayer::linux_open_dsp_device (int channels, int srate)
 {	int fd, stereo, temp, error ;
@@ -196,14 +249,14 @@ int SoundPlayer::linux_open_dsp_device (int channels, int srate)
 
         return 	fd ;
 } /* linux_open_dsp_device */
-
+#endif
 
 /*!
     \fn SoundPlayer::run
  */
 void SoundPlayer::run()
 {
-
+    #ifdef HAS_OSS
     if(output==OUTPUT_DEVDSP){
         while(!closeV){
             if(playing){
@@ -217,6 +270,23 @@ void SoundPlayer::run()
             }
         }
     }
+    #endif
+    #ifdef HAS_ALSA
+    if(output==OUTPUT_ALSA){
+        while(!closeV){
+            if(playing){
+                playALSA();
+                if(playing) position=leftLoop;
+                snd_pcm_prepare(alsa_pcm_handle);
+            }else if(playFromFileV){
+                if(playing) stopPlay();
+                playFromFileThreaded();
+            }else{
+                msleep(100);
+            }
+        }
+    }
+    #endif
 }
 
 
@@ -247,8 +317,10 @@ bool SoundPlayer::isPlaying()
 void SoundPlayer::setPlaying(bool _pl)
 {
     playing=_pl;
-    if(output==OUTPUT_PORTAUDIO && playing) startPAStream();
-    else if(output==OUTPUT_PORTAUDIO && !playing) stopPAStream();
+    #ifdef HAS_PORTAUDIO
+        if(output==OUTPUT_PORTAUDIO && playing) startPAStream();
+        else if(output==OUTPUT_PORTAUDIO && !playing) stopPAStream();
+    #endif
 }
 
 
@@ -285,6 +357,11 @@ void SoundPlayer::setPlayingOnce(bool po)
     playingOnce=po;
 }
 
+
+
+
+
+#ifdef HAS_JACK
 int SoundPlayer::closeJack ()
 {
     jack_deactivate (jack_handle);
@@ -336,22 +413,6 @@ int SoundPlayer::connect_jack()
 }
 
 
-/*!
-    \fn SoundPlayer::connect_devdsp()
- */
-int SoundPlayer::connect_devdsp()
-{
-        fprintf (stderr, "Querying /dev/dsp.\n");
-        //devdsp_audio_device = linux_open_dsp_device (soundManager->getChannels(),
-        //                                                                soundManager->getRate()) ;
-        devdsp_audio_device = linux_open_dsp_device (2,44100) ;
-        if(devdsp_audio_device){
-            output=OUTPUT_DEVDSP;
-            fprintf (stderr, "Connected to /dev/dsp.\n");
-            return 0;
-            bufferLR=new short[buffer_len*2];
-        }else return 1;
-}
 
 int SoundPlayer::jack_static_callback (jack_nframes_t nframes, void *arg)
 {
@@ -377,11 +438,11 @@ int SoundPlayer::jack_callback (jack_nframes_t nframes)
         float* buffL=new float[nframes];
         float* buffR=new float[nframes];
         float* buff=new float[nframes*2];
-        long length=sfinfoForPreview.frames;
-        if(position>=0 && position+buffer_len<length){
+        unsigned long length=sfinfoForPreview.frames;
+        if(position+buffer_len<length){
             sf_readf_float(previewFile, buff,(sf_count_t)(nframes));
             int count=0;
-            for (int i=0;i<nframes;i++){
+            for (uint i=0;i<nframes;i++){
                 buffL[i]=buff[count++];
                 buffR[i]=buff[count++];
             }
@@ -402,7 +463,6 @@ int SoundPlayer::jack_callback (jack_nframes_t nframes)
             return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
         if(isPlaying()){
             Q_ASSERT(position<soundManager->getFrames());
-            Q_ASSERT(position>=0);
             bufferPointerL=&soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT)[position];
             bufferPointerR=&soundManager->getRightChannel(SoundHolder::FOR_OUTPUT)[position];
             if(position+nframes<getRightLoop()){
@@ -443,13 +503,37 @@ int SoundPlayer::jack_callback (jack_nframes_t nframes)
 }
 
 
+/*!
+    \fn SoundPlayer::setJackNotPlaying()
+ */
+int SoundPlayer::setJackNotPlaying(jack_default_audio_sample_t * p, jack_nframes_t nframes)
+{
+    memset (p, 0, sizeof(jack_default_audio_sample_t) * nframes);
+    return 0;
+}
+
+#endif
+
 /*!
     \fn SoundPlayer::init()
  */
 void SoundPlayer::init()
 {
 
-    if(connect_jack() && connect_portaudio() && connect_devdsp()){
+    if(
+       #ifdef HAS_JACK
+        connect_jack() &&
+       #endif
+       #ifdef HAS_ALSA 
+        connect_alsa() &&
+       #endif
+       #ifdef HAS_PORTAUDIO 
+        connect_portaudio() &&
+       #endif
+       #ifdef HAS_OSS
+        connect_devdsp() &&
+       #endif
+        true){
         output=OUTPUT_NONE;
         std::cout<<"Oh, anyways we need no sound..Aren't we?\n";
     }
@@ -468,8 +552,8 @@ int SoundPlayer::getOutputType(){
     return output;
 }
 
-void SoundPlayer::cropped(long start,long end){
-    long temppposition=0;
+void SoundPlayer::cropped(unsigned long start, unsigned long end){
+    unsigned long temppposition=0;
     if(position<start) temppposition=0;
     else if(position>end) temppposition=soundManager->getFrames()-1;
     else temppposition=position-start;
@@ -488,14 +572,6 @@ void SoundPlayer::cropped(long start,long end){
 }
 
 
-/*!
-    \fn SoundPlayer::setJackNotPlaying()
- */
-int SoundPlayer::setJackNotPlaying(jack_default_audio_sample_t * p, jack_nframes_t nframes)
-{
-    memset (p, 0, sizeof(jack_default_audio_sample_t) * nframes);
-    return 0;
-}
 
 
 /*!
@@ -523,10 +599,13 @@ void SoundPlayer::playFromFile(QString path)
 {
     playFromFileV=TRUE;
     playFromFilePath=path;
-    //position=0;
-    if(output==OUTPUT_JACK || output==OUTPUT_PORTAUDIO) 
         previewFile= sf_open(path, SFM_READ, &sfinfoForPreview);
-    if(output==OUTPUT_PORTAUDIO) startPAStream();
+    #ifdef HAS_ALSA
+        if(output==OUTPUT_ALSA) snd_pcm_prepare(alsa_pcm_handle);
+    #endif
+    #ifdef HAS_PORTAUDIO
+        if(output==OUTPUT_PORTAUDIO) startPAStream();
+    #endif
 }
 
 /*!
@@ -534,17 +613,13 @@ void SoundPlayer::playFromFile(QString path)
  */
 void SoundPlayer::playFromFileThreaded()
 {
-    long length=0;
-    short* buff=new short[buffer_len*2];
-    SF_INFO sfinfo;
-    SNDFILE* tmp_sndfile= sf_open(playFromFilePath, SFM_READ, &sfinfo);
-    if (!tmp_sndfile) return;
-    sf_seek (tmp_sndfile, (sf_count_t)position, SEEK_SET);
-    length=sfinfo.frames;
+    unsigned long length=sfinfoForPreview.frames;
+    #ifdef HAS_OSS
     if(output==OUTPUT_DEVDSP){
+            short* buff=new short[buffer_len*2];
             while(position<length && playFromFileV){
-                if(position>0 && position+buffer_len<=length){
-                        sf_readf_short(tmp_sndfile, buff,(sf_count_t)(buffer_len));
+                if(position+buffer_len<=length){
+                        sf_readf_short(previewFile, buff,(sf_count_t)(buffer_len));
                         if (NOT_NATIVE_ENDIAN) swap_endian(buff, buffer_len*2);
                         write (devdsp_audio_device,buff,buffer_len*2* sizeof (short)) ;
                     }
@@ -552,9 +627,46 @@ void SoundPlayer::playFromFileThreaded()
                     if(playFromFileV) positionChanged(position);
             }
             zaparr(buff);
-            sf_close(tmp_sndfile);
-            if(playFromFileV) stopPlayFromFile();
+           if(playFromFileV) stopPlayFromFile();
+            else playFromFileV=FALSE;
         }
+    #endif
+    #ifdef HAS_ALSA
+    if(output==OUTPUT_ALSA){
+            bool pcm_return=0;
+            float* buff=new float[alsa_periodsize];
+            float* buffL=new float[alsa_frames];
+            float* buffR=new float[alsa_frames];
+            float** buffLR=new float*[2];
+            while(position<length && playFromFileV){
+                if(position+alsa_frames<=length){
+                        sf_readf_float(previewFile, buff,(sf_count_t)(alsa_frames));
+                        int count=0;
+                        for (int i=0;i<alsa_frames;i++){
+                            buffL[i]=buff[count++];
+                            buffR[i]=buff[count++];
+                        }
+                        
+                        buffLR[0]=buffL;
+                        buffLR[1]=buffR;
+                        pcm_return=snd_pcm_writen(alsa_pcm_handle,(void**) buffLR, alsa_frames);
+                        if(pcm_return<0){
+                            snd_pcm_prepare(alsa_pcm_handle);
+                            fprintf(stderr, "xrun !\n");
+                        }
+                    }
+                    position+=alsa_frames;
+                    if(playFromFileV) positionChanged(position);
+            }
+            snd_pcm_drop(alsa_pcm_handle);
+            zaparr(buff);
+            zaparr(buffL);
+            zaparr(buffR);
+            zaparr(buffLR);
+            if(playFromFileV) stopPlayFromFile();
+            else playFromFileV=FALSE;
+        }
+    #endif
 }
 
 
@@ -567,8 +679,13 @@ void SoundPlayer::stopPlayFromFile()
     playing=FALSE;
     position=0;
     positionChanged(position);
-    if(output==OUTPUT_PORTAUDIO) stopPAStream();
-    if(output==OUTPUT_JACK || output==OUTPUT_PORTAUDIO) sf_close(previewFile);
+    #ifdef HAS_PORTAUDIO
+        if(output==OUTPUT_PORTAUDIO) stopPAStream();
+    #endif
+    #ifdef HAS_ALSA
+        if(output==OUTPUT_ALSA) snd_pcm_drop(alsa_pcm_handle);
+    #endif
+    sf_close(previewFile);
 }
 
 /*!
@@ -577,7 +694,12 @@ void SoundPlayer::stopPlayFromFile()
 void SoundPlayer::pausePlayFromFile()
 {
     playFromFileV=FALSE;
-    if(output==OUTPUT_PORTAUDIO) stopPAStream(); 
+    #ifdef HAS_ALSA
+        if(output==OUTPUT_ALSA) snd_pcm_drop(alsa_pcm_handle);
+    #endif
+    #ifdef HAS_PORTAUDIO
+        if(output==OUTPUT_PORTAUDIO) stopPAStream(); 
+    #endif
     //if(output==OUTPUT_JACK) sf_close(previewFile);
 }
 
@@ -588,11 +710,11 @@ void SoundPlayer::pausePlayFromFile()
  */
 void SoundPlayer::resetLoop()
 {
-    if(savedLeftLoop!=-1 || savedRightLoop!=-1){
+    if(savedLeftLoop!=ULONG_MAX || savedRightLoop!=ULONG_MAX){
         leftLoop=savedLeftLoop;
         rightLoop=savedRightLoop;
-        savedLeftLoop=-1;
-        savedRightLoop=-1;
+        savedLeftLoop=ULONG_MAX;
+        savedRightLoop=ULONG_MAX;
     }
 }
 
@@ -602,6 +724,9 @@ void SoundPlayer::resetLoop()
  */
 void SoundPlayer::setPosition(long pos)
 {
+    if (playFromFileV){
+        sf_seek (previewFile, (sf_count_t)pos, SEEK_SET);
+    }
     positionChanged(pos);
 }
 
@@ -627,24 +752,24 @@ void SoundPlayer::removeSoundMonitor(QObject* sm)
     }
 }
 
-
+#ifdef HAS_PORTAUDIO
 /*!
     \fn SoundPlayer::portaudio_callback( const void *inputBuffer, void *outputBuffer,
                      unsigned long framesPerBuffer,
                      const PaStreamCallbackTimeInfo* timeInfo,
                      PaStreamCallbackFlags statusFlags)
  */
-int SoundPlayer::portaudio_callback( const void *inputBuffer, void *outputBuffer,
+int SoundPlayer::portaudio_callback( const void */*inputbuffer*/, void *outputBuffer,
                      unsigned long nframes,
-                     const PaStreamCallbackTimeInfo* timeInfo,
-                     PaStreamCallbackFlags statusFlags)
+                     const PaStreamCallbackTimeInfo* /*timeInfo*/,
+                     PaStreamCallbackFlags /*statusFlags*/)
 {
     if(!soundManager) return paComplete;
     if(playFromFileV){
         if (!previewFile) return paComplete;
-        long length=sfinfoForPreview.frames;
+        unsigned long length=sfinfoForPreview.frames;
         //cout<<"position: "<<position<<endl;
-        if(position>=0 && position+nframes<length){
+        if(position+nframes<length){
             //cout<<"position: "<<position<<endl;
             sf_readf_float(previewFile, (float*)outputBuffer,(sf_count_t)(nframes));
             position+=nframes;
@@ -660,9 +785,10 @@ int SoundPlayer::portaudio_callback( const void *inputBuffer, void *outputBuffer
     }else{
         if(isPlaying()){
                 if(position+nframes<getRightLoop()){
+                
                     soundManager->exportMultiplexedBuffer((float*)outputBuffer,
                                             position,
-                                            position+nframes);
+                                            position+long(nframes));
                     position+=nframes;
                 }else{
                     soundManager->exportMultiplexedBuffer((float*)outputBuffer,
@@ -794,25 +920,263 @@ int SoundPlayer::stopPAStream()
     }
     return 0;
 }
+#endif
+
+#ifdef HAS_ALSA
+QString SoundPlayer::getAlsaPcmName(){
+    return QString(alsa_pcm_name);
+}
+
+int SoundPlayer::setAlsaParameters(QString name, int frames, int periods, int rate){
+    alsa_rate = rate;
+    alsa_periods = periods;
+    alsa_periodsize = frames<<2;
+    QString nameQS=QStringList::split(" ",name).first();
+    alsa_pcm_name = (char*)nameQS.latin1();
+    if(alsa_pcm_handle!=0) snd_pcm_close(alsa_pcm_handle);
+    cout<<"device: "<<alsa_pcm_name<<" rate: "<<alsa_rate<<" periods: "<<
+            alsa_periods<<" period size: "<<alsa_periodsize<<" frames: "<<frames<<"\n";
+    int ret=0;
+    if((ret=connect_alsa())){
+        output=OUTPUT_NONE;
+    }
+    return ret;
+}
 
 
-long SoundPlayer::getRightLoop() const {
+QStringList SoundPlayer::getAlsaInterfaces(){
+                QStringList outputList;
+    // Enumerate the ALSA cards and PCM harfware devices...
+		snd_ctl_t *handle;
+		snd_ctl_card_info_t *info;
+		snd_pcm_info_t *pcminfo;
+		snd_ctl_card_info_alloca(&info);
+		snd_pcm_info_alloca(&pcminfo);
+		int iCard = -1;
+		int iCards = 0;
+		QString sName, sText;
+		while (snd_card_next(&iCard) >= 0 && iCard >= 0) {
+			sName = "hw:" + QString::number(iCard);
+			if (snd_ctl_open(&handle, sName.latin1(), 0) >= 0
+				&& snd_ctl_card_info(handle, info) >= 0) {
+
+				if (iCards > 0){
+					outputList.append("----------------------");
+			        }
+			         
+				sText  = sName + " [";
+				sText += QString(snd_ctl_card_info_get_name(info))+"]";
+				outputList.append("plug"+sText);
+				int iDevice = -1;
+				while (snd_ctl_pcm_next_device(handle, &iDevice) >= 0
+					&& iDevice >= 0) {
+					snd_pcm_info_set_device(pcminfo, iDevice);
+					snd_pcm_info_set_subdevice(pcminfo, 0);
+					snd_pcm_info_set_stream(pcminfo,SND_PCM_STREAM_PLAYBACK);
+					if (snd_ctl_pcm_info(handle, pcminfo) >= 0) {
+						sText  = sName + ',' + QString::number(iDevice) + " [";
+						sText += QString(snd_pcm_info_get_name(pcminfo))+"]";
+						outputList.append("plug"+sText);
+					}
+				}
+				snd_ctl_close(handle);
+				++iCards;
+			}
+		}
+		qHeapSort(outputList);
+		return outputList;
+}
+
+/*!
+    \fn SoundPlayer::connect_alsa()
+ */
+int SoundPlayer::connect_alsa()
+{
+    std::cout<<"Querying Alsa...\n";
+    snd_pcm_stream_t alsa_stream = SND_PCM_STREAM_PLAYBACK;
+    /* Init pcm_name. Of course, later you */
+    /* will make this configurable ;-)     */
+  
+    /* Allocate the snd_pcm_hw_params_t structure on the stack. */
+    snd_pcm_hw_params_alloca(&alsa_hwparams);
+    
+     /* Allocate the snd_pcm_sw_params_t structure on the stack. */
+    snd_pcm_sw_params_alloca(&alsa_swparams);
+    
+    /* Open PCM. The last parameter of this function is the mode. */
+    /* If this is set to 0, the standard mode is used. Possible   */
+    /* other values are SND_PCM_NONBLOCK and SND_PCM_ASYNC.       */ 
+    /* If SND_PCM_NONBLOCK is used, read / write access to the    */
+    /* PCM device will return immediately. If SND_PCM_ASYNC is    */
+    /* specified, SIGIO will be emitted whenever a period has     */
+    /* been completely processed by the soundcard.                */
+    if (snd_pcm_open(&alsa_pcm_handle, alsa_pcm_name, alsa_stream, 0) < 0) {
+      fprintf(stderr, "Error opening PCM device %s\n", alsa_pcm_name);
+      return(-1);
+    }
+    /* Init hwparams with full configuration space */
+    if (snd_pcm_hw_params_any(alsa_pcm_handle, alsa_hwparams) < 0) {
+      fprintf(stderr, "Can not configure this PCM device.\n");
+      return(-2);
+    }
+    
+     //snd_pcm_sw_params_set_start_threshold (alsa_pcm_handle, alsa_swparams, 0);
+     
+     if (snd_pcm_hw_params_set_access(alsa_pcm_handle, alsa_hwparams, SND_PCM_ACCESS_RW_NONINTERLEAVED) < 0) {
+      fprintf(stderr, "Error setting access.\n");
+      return(-3);
+    }
+    
+    /* Set sample format */
+    if (NOT_NATIVE_ENDIAN) {
+        if (snd_pcm_hw_params_set_format(alsa_pcm_handle, alsa_hwparams, SND_PCM_FORMAT_FLOAT_BE) < 0) {
+            fprintf(stderr, "Error setting format.\n");
+            return(-4);
+        }
+    }else{
+        if (snd_pcm_hw_params_set_format(alsa_pcm_handle, alsa_hwparams, SND_PCM_FORMAT_FLOAT_LE) < 0) {
+            fprintf(stderr, "Error setting format.\n");
+            return(-4);
+        }
+    }
+    
+    /* Set sample rate. If the exact rate is not supported */
+    /* by the hardware, use nearest possible rate.         */ 
+    alsa_exact_rate = alsa_rate;
+    if (snd_pcm_hw_params_set_rate_near(alsa_pcm_handle, alsa_hwparams, &alsa_exact_rate, 0) < 0) {
+      fprintf(stderr, "Error setting rate.\n");
+      return(-5);
+    }
+    if (alsa_rate != alsa_exact_rate) {
+      fprintf(stderr, "The rate %d Hz is not supported by your hardware.\n==> Using %d Hz instead.\n", alsa_rate, alsa_exact_rate);
+    }
+    
+    /* Set number of channels */
+    if (snd_pcm_hw_params_set_channels(alsa_pcm_handle, alsa_hwparams, 2) < 0) {
+      fprintf(stderr, "Error setting channels.\n");
+      return(-6);
+    }
+
+    /* Set number of periods. Periods used to be called fragments. */ 
+    if (snd_pcm_hw_params_set_periods(alsa_pcm_handle, alsa_hwparams, alsa_periods, 0) < 0) {
+      fprintf(stderr, "Error setting periods.\n");
+      return(-7);
+    }
+    
+    /* Set buffer size (in frames). The resulting latency is given by */
+    /* latency = periodsize * periods / (rate * bytes_per_frame)     */
+    if (snd_pcm_hw_params_set_buffer_size(alsa_pcm_handle, alsa_hwparams, (alsa_periodsize * alsa_periods)>>2) < 0) {
+      fprintf(stderr, "Error setting buffersize.\n");
+      return(-8);
+    }
+    
+    /* Apply HW parameter settings to */
+    /* PCM device and prepare device  */
+    if (snd_pcm_hw_params(alsa_pcm_handle, alsa_hwparams) < 0) {
+      fprintf(stderr, "Error setting HW params.\n");
+      return(-9);
+    }
+    
+    /* Apply SW parameter settings to */
+    /* PCM device and prepare device  */
+    /*if (snd_pcm_sw_params(alsa_pcm_handle, alsa_swparams) < 0) {
+      fprintf(stderr, "Error setting HW params.\n");
+      return(-10);
+    }*/
+    
+    alsa_buffer_size=((alsa_periodsize * alsa_periods)>>2);
+    alsa_frames=alsa_periodsize>>2;
+    output=OUTPUT_ALSA;
+    bufferLR_F=new float[alsa_periodsize];
+    float latency = alsa_periodsize * float(alsa_periods) / float(((alsa_rate/1000) * 8 /*bytes_per_frame*/));
+    cout<<"Initialised ALSA - "<<" device: "<<alsa_pcm_name<<" rate: "<<alsa_exact_rate<<" channels: 2 periods: "<<alsa_periods<<" buffer size: "<<
+        alsa_buffer_size<<" frames: "<<alsa_frames<<" period size: "<<alsa_periodsize<<" latency: "<<latency<<" ms\n";
+    return 0;
+}
+
+void  SoundPlayer::playALSA(){
+    while(position<rightLoop && playing){
+            int pcm_return=0;
+            if(position+alsa_frames<rightLoop){
+                bufferPointersLR[0]=&soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT)[position];
+                bufferPointersLR[1]=&soundManager->getRightChannel(SoundHolder::FOR_OUTPUT)[position];
+                pcm_return=snd_pcm_writen(alsa_pcm_handle,(void**)bufferPointersLR, alsa_frames);
+                if(pcm_return>0) position+=pcm_return;
+                
+                /*
+                soundManager->exportMultiplexedBuffer(bufferLR_F, position,position+alsa_frames);
+                pcm_return=snd_pcm_writei(alsa_pcm_handle, bufferLR_F, alsa_frames);
+                if(pcm_return>0) position+=pcm_return;
+                */
+            }else if (position<rightLoop){
+                /*
+                soundManager->exportMultiplexedBuffer(bufferLR_F, position,
+                        rightLoop);
+                */
+                 if(!playingOnce){
+                    bufferPointersLR[0]=&soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT)[position];
+                    bufferPointersLR[1]=&soundManager->getRightChannel(SoundHolder::FOR_OUTPUT)[position];
+                    pcm_return=snd_pcm_writen(alsa_pcm_handle,(void**)bufferPointersLR, rightLoop-position);
+                    bufferPointersLR[0]=&soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT)[leftLoop];
+                    bufferPointersLR[1]=&soundManager->getRightChannel(SoundHolder::FOR_OUTPUT)[leftLoop];
+                    pcm_return=snd_pcm_writen(alsa_pcm_handle,(void**)bufferPointersLR,(alsa_frames-(rightLoop-position)));
+                    
+                    /*
+                    soundManager->exportMultiplexedBuffer(&bufferLR_F[rightLoop-position], leftLoop,
+                            leftLoop+(alsa_frames-(rightLoop-position)));
+                    */
+                    //cout<<"alsa_frames: "<<alsa_frames<<" (alsa_frames-(rightLoop-position)): "<<(alsa_frames-(rightLoop-position));
+                    //cout<< " rightLoop-position : "<<rightLoop-position<<"\n";
+                    //pcm_return=snd_pcm_writen(alsa_pcm_handle, bufferLR_F, alsa_frames);
+                    if(pcm_return>0) position=leftLoop+(pcm_return-(rightLoop-position));
+                 }else{
+                    bufferPointersLR[0]=&soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT)[position];
+                    bufferPointersLR[1]=&soundManager->getRightChannel(SoundHolder::FOR_OUTPUT)[position];
+                    pcm_return=snd_pcm_writen(alsa_pcm_handle,(void**)bufferPointersLR, rightLoop-position);
+                    snd_pcm_drain(alsa_pcm_handle);
+                    //memset(&bufferLR_F[rightLoop-position],0,(alsa_frames-(rightLoop-position)));
+                    //pcm_return=snd_pcm_writei(alsa_pcm_handle, bufferLR_F, alsa_frames);
+                    position=rightLoop;
+                 }
+            }else position=rightLoop;
+            if(isPlaying()) positionChanged(position);
+            if(pcm_return<0){
+                snd_pcm_prepare(alsa_pcm_handle);
+                //cout<<"-EBADFD: "<<-EBADFD<<" -EPIPE: "<<-EPIPE<<" -ESTRPIPE: "<<-ESTRPIPE<<" -EAGAIN "<<-EAGAIN;
+                //cout<<" -EINVAL: "<<-EINVAL<<" -EIO: "<<-EIO<<"\n";
+                cout<<"xrun !   ("<<pcm_return<<")\n";
+            }
+        }
+        if(playingOnce){
+            this->stopPlay();
+            playingOnce=FALSE;
+            resetLoop();
+        }
+}
+#endif
+
+unsigned long SoundPlayer::getRightLoop() const {
     if (position<rightLoop) return rightLoop;
     else return soundManager->getFrames();
 }
 
 
-void SoundPlayer::setRightLoop(const long& theValue) {
+void SoundPlayer::setRightLoop(const unsigned long& theValue) {
     rightLoop = theValue;
 }
 
 
-long SoundPlayer::getLeftLoop() const {
+unsigned long SoundPlayer::getLeftLoop() const {
     if (position<rightLoop) return leftLoop;
     else return 0;
 }
 
 
-void SoundPlayer::setLeftLoop(const long& theValue) {
+void SoundPlayer::setLeftLoop(const unsigned long& theValue) {
     leftLoop = theValue;
 }
+
+
+void SoundPlayer::setMainApplication(QApplication* theValue) {
+    mainApplication = theValue;
+}
