@@ -49,14 +49,14 @@ SpectrumView::~SpectrumView()
     zaparr(subband_energies);
 }
 
-void SpectrumView::showEvent(QShowEvent* ev){
+void SpectrumView::showEvent(QShowEvent* /*ev*/){
     if(contentsRect().size().isValid()){
         //subband_number=round(contentsRect().width()/10);
         //steps=round(contentsRect().height()/3);
     }
 }
 
-void SpectrumView::paintEvent(QPaintEvent* ev){
+void SpectrumView::paintEvent(QPaintEvent* /*ev*/){
     static QPixmap pixmap;
     bool resized=false;  //force repaint on resize even if between frames.
     if(pixmap.size()!=contentsRect().size() && contentsRect().size().isValid()){
@@ -71,7 +71,7 @@ void SpectrumView::paintEvent(QPaintEvent* ev){
         int y=contentsRect().y();
         if(playing){
             float* lchannel=&soundManager->getLeftChannel()[position];
-            float* rchannel=&soundManager->getRightChannel()[position];
+            //float* rchannel=&soundManager->getRightChannel()[position];
 
 
 
@@ -85,14 +85,15 @@ void SpectrumView::paintEvent(QPaintEvent* ev){
             fftwf_execute(p);
             //fftw_execute(p, in2, out2); //compute fft
 
-            int subbandSize=round(float(frames/2)/float(subband_number));
+            int subbandSize=int(round(float(frames/2)/float(subband_number)));
             float* subband_buffer=new float[frames/2];
-            for (int j=1;j<((frames+1)/2)-1;j++){
+            for (int j=1;j<frames/2-1;j++){
                 subband_buffer[j]=  sqrt(pow(out[j][0],2) + pow(out[j][1],2));
             }
 
             //compute the energy of n subbands
-            for (int i2=1;i2<((frames+1)/2)-1;i2+=subbandSize){
+            int reminder=(frames/2-1) % subbandSize;
+            for (int i2=0;i2<frames/2-1-reminder;i2+=subbandSize){
                 double subbandE=0;
 
                 for (int i3=i2;i3<i2+subbandSize;i3++){
@@ -115,11 +116,11 @@ void SpectrumView::paintEvent(QPaintEvent* ev){
         QPainter p(&pixmap);
         p.fillRect(0,0,contentsRect().width(),contentsRect().height(),QBrush::black);
         float totalBandWidth=float(contentsRect().width())/float(subband_number);
-        int rectWidth=round(round(totalBandWidth)*0.5f);
-        int spaceH=round(round(totalBandWidth)*0.5f);
+        int rectWidth=int(round(round(totalBandWidth)*0.5f));
+        int spaceH=int(round(round(totalBandWidth)*0.5f));
         float totalBandHeight=float(contentsRect().height())/float(steps);
-        int rectHeight=round(round(totalBandHeight)*0.5f);
-        int spaceV=round(round(totalBandHeight)*0.5f);
+        int rectHeight=int(round(round(totalBandHeight)*0.5f));
+        int spaceV=int(round(round(totalBandHeight)*0.5f));
 
         QColor color(0,255,0);
         QColor fadedColor(0,255,0);
@@ -140,7 +141,7 @@ void SpectrumView::paintEvent(QPaintEvent* ev){
                     g=g>255?255:g;
                 }
                 color.setRgb(r,g,b);
-                fadedColor.setRgb(float(r)*0.2,float(g)*0.2,float(b)*0.2);
+                fadedColor.setRgb(int(float(r)*0.2),int(float(g)*0.2),int(float(b)*0.2));
                 brush.setColor(color);
                 fadedBrush.setColor(fadedColor);
                 if(subband_energies[i]>=double(j)/double(steps)){
@@ -183,8 +184,8 @@ void SpectrumView::setSoundManager(SoundManager* sm)
 void SpectrumView::resizeEvent(QResizeEvent* ev)
 {
     if(ev->size().isValid()){
-        subband_number=round(ev->size().width()/10);
-        steps=round(ev->size().height()/3);
+        subband_number=int(round(ev->size().width()/10));
+        steps=int(round(ev->size().height()/3));
         if(subband_number>0){
             if(subband_energies) zaparr(subband_energies);
             subband_energies=new float[subband_number];
